# 爬虫名注册以及配置 配置优先度高于custom_settings 仅配置中间件和管道 该文件未配置的取custom_settings
# 说明: 爬虫模块<str> : [爬虫名<str>, 爬虫中间件<tuple>,下载中间件<tuple>,管道<tuple>,拓展中间件<tuple>]
# 示例: "pyppeteer_demo.crawl":["zz_rohmcrawl",("stickyitem" ..),("ua","proxy","pyppeteer" ..),("file","file_bak","img","img_bak","bloom","mysql","mongo","rabbit","fctexcel","dpfexcel","kafka" ..),("closespider"，"mailer" ..)],

zhizhong = {
}

yongjie = {
}

xuqi = {
}

juncheng = {
}

yonghao = {
}

siming = {
}

fengchao = {
}

runtao = {
}

yijia = {
}

shixi = {
}